require 'sudoku/model/grid'

module Sudoku
  # This class provides method for solving or checking solution of the
  # Sudoku game.
  class Solver
    # Will initialize this solver with given grid. Exception is thrown
    # if the given grid is not kind of Sudoku::Grid
    def initialize(grid)
      fail ArgumentError,
           'Invalid grid passed to solver' unless grid.is_a?(Grid)
      @grid = grid
    end

    # Will check if the sudoku rules are violated, but the 0 value is
    # ignored. Returns true if violated, false otherwise.
    def rules_violated?
      rows_violated? || cols_violated? || areas_violated?
    end

    # Will return true, if all cells of the grid are filled and no
    # Sudoku rule is violated.
    def solved?
      !rules_violated? && grid_filled?
    end

    # Will return true, if the grid is filled, which means, that all
    # of the values are other than 0. False otherwise.
    def grid_filled?
      @grid.filled?
    end

    # Will return true, if all rows are correctly solved.
    def rows_violated?
      (0..8).to_a.collect { |x|cells_violated?(@grid.get_row(x)) }.any?
    end

    # Will return true, if all cols are correctly solved.
    def cols_violated?
      (0..8).to_a.collect { |x|cells_violated?(@grid.get_col(x)) }.any?
    end

    # Will return true, if given set of cells does not contain
    # duplicities in values other than 0.
    # * +cells+ set of cells to check.
    def cells_violated?(cells)
      values = cells.select { |x| x.value != 0 }.collect(&:value)
      values.uniq.size != values.size
    end

    # Will return true, if in some area of the grid are duplicit or
    # illegal numbers. Returns false otherwise.
    def areas_violated?
      [(0..2), (3..5), (6..8)].repeated_permutation(2).each do |x, y|
        cells = @grid.cells.select { |cell| cell.in_area(x, y) }
        return true if cells_violated?(cells)
      end
      false
    end
  end
end
